package com.sleepycat.je.dbi;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.incomp.INCompressor;
import com.sleepycat.je.tree.BIN;
import com.sleepycat.je.tree.BINReference;

public privileged aspect CursorVerifier {
	
	public void EnvironmentImpl.verifyCursors() throws DatabaseException {

		inCompressor.verifyCursors();
	}
	
	
	public synchronized void INCompressor.verifyCursors() throws DatabaseException {

		/*
		 * Environment may have been closed. If so, then our job here is done.
		 */
		if (env.isClosed()) {
			return;
		}

		/*
		 * Use a snapshot to verify the cursors. This way we don't have to hold
		 * a latch while verify takes locks.
		 */
		List queueSnapshot = null;
		synchronized (binRefQueueSync) {
			queueSnapshot = new ArrayList(binRefQueue.values());
		}

		/* Use local caching to reduce DbTree.getDb overhead. */
		Map dbCache = new HashMap();

		Iterator it = queueSnapshot.iterator();
		while (it.hasNext()) {
			BINReference binRef = (BINReference) it.next();
			DatabaseImpl db = env.getDbMapTree().getDbCaching(binRef.getDatabaseId(),
					lockTimeout, dbCache);
			BIN bin = searchForBIN(db, binRef);
			if (bin != null) {
				bin.verifyCursors();
				//Lck bin.releaseLatch();
			}
		}
	}
	
	/**
	 * For each cursor in this BIN's cursor set, ensure that the cursor is
	 * actually referring to this BIN.
	 */
	public void BIN.verifyCursors() {
		if (cursorSet != null) {
			Iterator iter = cursorSet.iterator();
			while (iter.hasNext()) {
				CursorImpl cursor = (CursorImpl) iter.next();
				if (getCursorBINToBeRemoved(cursor) != this) {
					BIN cBin = getCursorBIN(cursor);
					assert cBin == this;
				}
			}
		}
	}
}
