package com.sleepycat.je.dbi;

import java.io.PrintStream;
import java.util.HashSet;
import java.util.Set;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.tree.BIN;
import com.sleepycat.je.tree.DBIN;
import com.sleepycat.je.tree.DIN;
import com.sleepycat.je.tree.DupCountLN;
import com.sleepycat.je.tree.IN;
import com.sleepycat.je.tree.Node;
import com.sleepycat.je.tree.TreeVisitor;

public class TreeVerifier implements TreeVisitor {

	PrintStream progressStream;

	private Set inNodeIdsSeen = new HashSet();

	private Set binNodeIdsSeen = new HashSet();

	private Set dinNodeIdsSeen = new HashSet();

	private Set dbinNodeIdsSeen = new HashSet();

	public TreeVerifier(PrintStream progressStream) {
		this.progressStream = progressStream;
	}

	public void verifyNode(Node node) {

		try {
			node.verify(null);
		} catch (DatabaseException INE) {
			progressStream.println(INE);
		}
	}

	public void processIN(IN node, Long nid, int level) {
		if (inNodeIdsSeen.add(nid)) {
			verifyNode(node);
		}
	}

	public void processBIN(BIN node, Long nid, int level) {
		if (binNodeIdsSeen.add(nid)) {
			verifyNode(node);
		}
	}

	public void processDIN(DIN node, Long nid, int level) {
		if (dinNodeIdsSeen.add(nid)) {
			verifyNode(node);
		}
	}

	public void processDBIN(DBIN node, Long nid, int level) {
		if (dbinNodeIdsSeen.add(nid)) {
			verifyNode(node);
		}
	}

	public void processDupCountLN(DupCountLN node, Long nid) {
		verifyNode(node);
	}

	public void incrementLNCount() {
	}

	public void incrementDeletedLNCount() {
	}
}
