package com.sleepycat.je.dbi;

import java.util.Comparator;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.tree.BIN;
import com.sleepycat.je.tree.IN;
import com.sleepycat.je.tree.InconsistentNodeException;
import com.sleepycat.je.tree.Key;
import com.sleepycat.je.tree.Node;

public privileged aspect VerificationCode {
	public void Node.verify(byte[] maxKey) throws DatabaseException {
	}
	
	/**
	 * Check that the IN is in a valid state. For now, validity means that the
	 * keys are in sorted order and that there are more than 0 entries. maxKey,
	 * if non-null specifies that all keys in this node must be less than
	 * maxKey.
	 */
	public void IN.verify(byte[] maxKey) throws DatabaseException {
		// Lck slight change (exception on latch() is not cached) to receive
		// homogenious crosscuts
		try {
			Comparator userCompareToFcn = (databaseImpl == null ? null
					: getKeyComparator());

			byte[] key1 = null;
			for (int i = 1; i < nEntries; i++) {
				key1 = entryKeyVals[i];
				byte[] key2 = entryKeyVals[i - 1];

				int s = Key.compareKeys(key1, key2, userCompareToFcn);
				if (s <= 0) {
					throw new InconsistentNodeException("IN " + getNodeId()
							+ " key " + (i - 1) + " ("
							+ Key.dumpString(key2, 0) + ") and " + i + " ("
							+ Key.dumpString(key1, 0) + ") are out of order");
				}
			}

			boolean inconsistent = false;
			if (maxKey != null && key1 != null) {
				if (Key.compareKeys(key1, maxKey, userCompareToFcn) >= 0) {
					inconsistent = true;
				}
			}

			if (inconsistent) {
				throw new InconsistentNodeException("IN " + getNodeId()
						+ " has entry larger than next entry in parent.");
			}
		} catch (DatabaseException DE) {
			DE.printStackTrace(System.out);
		}
	}
	
	
	/*
	 * For debugging. Verify that a BINs cursor set refers to the BIN.
	 */
	void CursorImpl.verifyCursor(BIN bin) throws DatabaseException {

		if (!bin.getCursorSet().contains(this)) {
			throw new DatabaseException("BIN cursorSet is inconsistent.");
		}
	}
}
